@@ -27,9 +27,8 @@
 #include <string.h>
 #include <sys/stat.h>
 
-#include <glib/gi18n-lib.h>
-
 #include <libedataserver/e-data-server-util.h>
+#include <camel/camel-i18n.h>
 
 #include "camel-exchange-folder.h"
 #include "camel-exchange-search.h"
@@ -431,6 +430,8 @@ get_message_data (CamelFolder *folder, const char *uid, CamelException *ex)
 	return ba;
 }
 
+#define MAILING_LIST_HEADERS "X-MAILING-LIST X-LOOP LIST-ID LIST-POST MAILING-LIST ORIGINATOR X-LIST RETURN-PATH X-BEENTHERE "
+
 static CamelMimeMessage *
 get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 {
@@ -440,6 +441,8 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	CamelStreamFilter *filtered_stream;
 	CamelMimeFilter *crlffilter;
 	GByteArray *ba;
+	gchar **list_headers = NULL;
+	gboolean found_list = FALSE;
 
 	ba = get_message_data (folder, uid, ex);
 	if (!ba)
@@ -457,8 +460,26 @@ get_message (CamelFolder *folder, const char *uid, CamelException *ex)
 	camel_data_wrapper_construct_from_stream (CAMEL_DATA_WRAPPER (msg),
 						  CAMEL_STREAM (filtered_stream));
 	camel_object_unref (CAMEL_OBJECT (filtered_stream));
-
 	camel_mime_message_set_source (msg, exch->source);
+
+	if(camel_medium_get_header (CAMEL_MEDIUM (msg), "Sender")) {
+		list_headers = g_strsplit (MAILING_LIST_HEADERS, " ", 0);
+		if (list_headers) {
+			int i = 0;
+			while (list_headers[i]) {
+				if (camel_medium_get_header (CAMEL_MEDIUM (msg), list_headers[i])) {
+					found_list = TRUE;
+					break;
+				}
+				i++;
+			}
+			g_strfreev (list_headers);
+		}
+		
+		if (!found_list)
+			camel_medium_set_header (CAMEL_MEDIUM (msg), "X-Evolution-Mail-From-Delegate", "yes");
+	}
+
 	fix_broken_multipart_related (CAMEL_MIME_PART (msg));
 	return msg;
 }
@@ -750,7 +771,8 @@ find_parent (CamelExchangeFolder *exch, const char *thread_index)
 void
 camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 				   const char *uid, guint32 flags,
-				   guint32 size, const char *headers)
+				   guint32 size, const char *headers,
+				   const char *href)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfo *info;
@@ -793,6 +815,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	info->uid = g_strdup (uid);
 	einfo->info.flags = flags;
 	einfo->info.size = size;
+	einfo->href = g_strdup (href);
 
 	camel_folder_summary_add (folder->summary, info);
 
@@ -999,7 +1022,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 	CamelExchangeFolder *exch = (CamelExchangeFolder *)folder;
 	const char *short_name;
 	char *summary_file, *journal_file, *path;
-	GPtrArray *summary, *uids;
+	GPtrArray *summary, *uids, *hrefs;
 	GByteArray *flags;
 	guint32 folder_flags;
 	CamelMessageInfo *info;
@@ -1078,11 +1101,14 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		g_ptr_array_set_size (uids, summary->len);
 		flags = g_byte_array_new ();
 		g_byte_array_set_size (flags, summary->len);
+		hrefs = g_ptr_array_new ();
+		g_ptr_array_set_size (hrefs, summary->len);
 
 		for (i = 0; i < summary->len; i++) {
 			info = summary->pdata[i];
 			uids->pdata[i] = (char *)camel_message_info_uid (info);
 			flags->data[i] = ((CamelMessageInfoBase *)info)->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
+			hrefs->pdata[i] = ((CamelExchangeMessageInfo *)info)->href;
 			//camel_tag_list_free (&((CamelMessageInfoBase *)info)->user_tags);
 		}
 
@@ -1092,6 +1118,8 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 				      CAMEL_STUB_ARG_UINT32, create,
 				      CAMEL_STUB_ARG_STRINGARRAY, uids,
 				      CAMEL_STUB_ARG_BYTEARRAY, flags,
+				      CAMEL_STUB_ARG_STRINGARRAY, hrefs,
+				      CAMEL_STUB_ARG_UINT32, CAMEL_EXCHANGE_SUMMARY(folder->summary)->high_article_num,
 				      CAMEL_STUB_ARG_RETURN,
 				      CAMEL_STUB_ARG_UINT32, &folder_flags,
 				      CAMEL_STUB_ARG_STRING, &exch->source,
@@ -1099,6 +1127,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		camel_operation_end (NULL);
 		g_ptr_array_free (uids, TRUE);
 		g_byte_array_free (flags, TRUE);
+		g_ptr_array_free (hrefs, TRUE);
 		camel_folder_free_summary (folder, summary);
 		if (!ok)
 			return FALSE;
@@ -1106,6 +1135,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		if (len)
 			return TRUE;
 
+
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER)
 			folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
 		if (folder_flags & CAMEL_STUB_FOLDER_FILTER_JUNK)
